import setup

from lale.grammar import Grammar
from lale.lib.lale import Hyperopt
from lale.pretty_print import to_string
from sklearn.metrics import accuracy_score

from lale.lib.sklearn import MinMaxScaler as MinMax
from lale.lib.sklearn import StandardScaler as Standard
from lale.lib.sklearn import Normalizer as Norm
from lale.lib.sklearn import RobustScaler as Robust

from lale.lib.sklearn import PCA
from lale.lib.sklearn import PolynomialFeatures as PolyF
from lale.lib.sklearn import Nystroem as Nys

from lale.lib.sklearn import GaussianNB
from lale.lib.sklearn import GradientBoostingClassifier
from lale.lib.sklearn import KNeighborsClassifier as KNN
from lale.lib.sklearn import RandomForestClassifier
from lale.lib.sklearn import ExtraTreesClassifier
from lale.lib.sklearn import QuadraticDiscriminantAnalysis as QDA
from lale.lib.sklearn import PassiveAggressiveClassifier
from lale.lib.sklearn import DecisionTreeClassifier
from lale.lib.sklearn import LogisticRegression as LR
from lale.lib.sklearn import SVC

from lale.lib.xgboost import XGBClassifier as XGB
from lale.lib.lightgbm import LGBMClassifier as LGBM

from lale.lib.lale import ConcatFeatures as Concat
from lale.lib.lale import NoOp


g = Grammar()

g.start = g.process >> g.features >> g.model
g.process = g.process_elt | (g.process_elt & g.process) >> Concat
g.features = g.feature_elt \
           | ((g.feature_elt | g.classifier) & g.features) >> Concat
g.model = g.classifier

g.process_elt = NoOp | MinMax | Standard | Norm | Robust
g.feature_elt =  NoOp | PCA | PolyF | Nys
g.classifier = GaussianNB | GradientBoostingClassifier | KNN \
       | RandomForestClassifier | ExtraTreesClassifier | QDA \
       | PassiveAggressiveClassifier | DecisionTreeClassifier \
       | LR | XGB | LGBM | SVC

def grammar_trainer(data, args):
    (X_train, y_train), (X_test, y_test) = data
    generated = g.unfold(args.max_depth)
    trainer = Hyperopt(
        estimator=generated,
        cv=args.cv,
        max_evals=args.max_eval,
        scoring='accuracy',
        max_opt_time=args.max_opt_time,
        max_eval_time=args.max_eval_time)
    trained = trainer.fit(X_train, y_train)    
    y_hat = trained.predict(X_test)
    test_acc = str(accuracy_score(y_test, y_hat))
    return {
        'test_acc': str(accuracy_score(y_test, y_hat)),
        'pipeline': to_string(trained.get_pipeline(), show_imports=False)}
  
  
setup.run_experiments('grammar_tpot_1', grammar_trainer)
